/********************************************************************
* Copyright (C) 2003-2008 Texas Instruments Incorporated.
* All Rights Reserved
*********************************************************************
* file: cslr_bcp_tm.h
*
* Brief: This file contains the Register Description for bcp_tm
*
*********************************************************************/
#ifndef _CSLR_BCP_TM_MMR_H_
#define _CSLR_BCP_TM_MMR_H_

#include <ti/csl/cslr.h>

#include <ti/csl/tistdtypes.h>


/* Minimum unit = 1 byte */

/**************************************************************************\
* Register Overlay Structure for TM Special Interrupt Registers
\**************************************************************************/
typedef struct  {
    volatile Uint32 BCP_INTERRUPT_STATUS; /* BCP interrupt status for DSP 0 */
    volatile Uint32 BCP_HALT_ON_ERROR; /* BCP halt on error control register for DSP 0 */
} CSL_Bcp_tmTm_special_interrupt_registersRegs; /*  TM Special Interrupt Registers */

/**************************************************************************\
* Register Overlay Structure for TM Interrupt Registers
\**************************************************************************/
typedef struct  {
    volatile Uint32 TM_INTR_IRS; /* BCP Raw Status Register */
    volatile Uint32 TM_INTR_IRS_SET; /* BCP Set Raw Status Register */
    volatile Uint32 TM_INTR_IRS_CLR; /* BCP Clear Raw Status Register */
    volatile Uint32 TM_INTR_EN; /* BCP Enable TM_intr0 Register */
    volatile Uint32 TM_INTR_EN_SET; /* BCP Set Enable TM_intr0 Register */
    volatile Uint32 TM_INTR_EN_CLR; /* BCP Clear Enable TM_intr0 Register */
    volatile Uint32 TM_INTR_EN_STS; /* BCP Enabled TM_intr0 Register */
} CSL_Bcp_tmTm_interrupt_registersRegs; /* TM interrupt register group */

/**************************************************************************\
* Register Overlay Structure for tm Data Logger
\**************************************************************************/
typedef struct  {
    volatile Uint32 TM_DLG_GLOBAL_HDR; /* Data logger control Register */
    volatile Uint32 TM_DLG_TM_SW_TIMESTAMP; /* Data logger control Register */
    volatile Uint32 TM_DLG_HW_TIMESTAMP_ENG_ERR; /* Data logger control Register */
    volatile Uint8 RSVD0[4];
} CSL_Bcp_tmTm_data_loggerRegs; /* Data Logger */

/**************************************************************************\
* Register Overlay Structure
\**************************************************************************/
typedef struct  {
    volatile Uint32 BCP_PID; /* BCP PID register */
    volatile Uint32 BCP_SOFT_RESET; /* BCP soft reset  register */
    volatile Uint32 TM_HALT_CTRL; /* TM halt control register */
    volatile Uint32 TM_HALT_STATUS; /* TM halt status register */
    volatile Uint32 BCP_EMU_CLKSTOP_CTRL; /* BCP Emulation control register */
    volatile Uint32 BCP_EMU_CLKSTOP_STATUS; /* BCP emulation and clockstop status */
    volatile Uint32 BCP_DLG_SW_TIMESTAMP; /* BCP Datalogger Software Timestamp */
    volatile Uint32 BCP_DLG_HW_TIMESTAMP; /* BCP Datalogger Hardware Timestamp */
    volatile Uint32 BCP_DLG_HW_TIMESTAMP_CTRL; /* BCP Datalogger Hardware Timestamp ctrl */
    volatile Uint32 BCP_DLG_HOLD_RST; /* BCP DLG hold rst register */
    volatile Uint32 TM_CONTROL; /* TM control register */
    volatile Uint32 TM_TX_CDMAHP_READ_PRIORITY; /* TM TX CDMAHP read priority register */
    volatile Uint32 TM_TX_QFIFO_RD_DEST_SEL; /* TM TX QFIFO read dest sel and arbitrtion priority register */
    volatile Uint32 TM_RX_QFIFO_WR_ARB_PRIORITY_3_0; /* TM RX QFIFO write arb priority register for channels 0 to 3 */
    volatile Uint32 TM_RX_QFIFO_WR_ARB_PRIORITY_7_4; /* TM RX QFIFO write arb priority register for channels 7 to 4 */
    volatile Uint32 TM_RX_CDMAHP_WR_ARB_PRIORITY; /* TM RX CDMAHP write arb priority register */
    CSL_Bcp_tmTm_special_interrupt_registersRegs TM_SPECIAL_INTERRUPT_REGISTERS[4];
    volatile Uint32 BCP_EOI; /* BCP EOI register */
    volatile Uint32 CDMA_DESC_STARVE_STATUS; /* CDMAHP Descriptor Starve status register */
    volatile Uint32 CDMA_DESC_STARVE_CLEAR; /* CDMAHP Descriptor Starve Clear register */
    volatile Uint32 CDMA_DESC_STARVE_SET; /* CDMAHP Descriptor Starve Set register */
    volatile Uint32 CDMA_DESC_STARVE_INTR_SEL; /* CDMAHP Descriptor Starve intr sel register */
    volatile Uint8 RSVD0[12];
    CSL_Bcp_tmTm_interrupt_registersRegs TM_INTERRUPT_REGISTERS[4];
    volatile Uint32 TM_DATA_LOGGER_CTL; /* Data logger control Register */
    volatile Uint32 TM_DATA_LOGGER_STATUS; /* Datalogger Status Register */
    volatile Uint32 TM_GLOBAL_HDR; /* TM Global Header Register */
    volatile Uint8 RSVD1[4];
    volatile Uint32 FLOW_ID_TABLE_0[64]; /* TM Flow ID Table Register 0 */
    volatile Uint8 RSVD2[7680];
    CSL_Bcp_tmTm_data_loggerRegs TM_DATA_LOGGER[256];
} CSL_Bcp_tmRegs;

/**************************************************************************\
* Field Definition Macros
\**************************************************************************/

/* bcp_interrupt_status */

#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_TM_INTR_STATUS_MASK (0x00000001u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_TM_INTR_STATUS_SHIFT (0x00000000u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_TM_INTR_STATUS_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_DIO_INTR_STATUS_MASK (0x00000002u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_DIO_INTR_STATUS_SHIFT (0x00000001u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_DIO_INTR_STATUS_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_CRC_INTR_STATUS_MASK (0x00000004u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_CRC_INTR_STATUS_SHIFT (0x00000002u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_CRC_INTR_STATUS_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_ENC_INTR_STATUS_MASK (0x00000008u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_ENC_INTR_STATUS_SHIFT (0x00000003u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_ENC_INTR_STATUS_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_RM_INTR_STATUS_MASK (0x00000010u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_RM_INTR_STATUS_SHIFT (0x00000004u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_RM_INTR_STATUS_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_MOD_INTR_STATUS_MASK (0x00000020u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_MOD_INTR_STATUS_SHIFT (0x00000005u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_MOD_INTR_STATUS_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_INT_INTR_STATUS_MASK (0x00000040u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_INT_INTR_STATUS_SHIFT (0x00000006u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_INT_INTR_STATUS_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_COR_INTR_STATUS_MASK (0x00000080u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_COR_INTR_STATUS_SHIFT (0x00000007u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_COR_INTR_STATUS_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_SSL_INTR_STATUS_MASK (0x00000100u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_SSL_INTR_STATUS_SHIFT (0x00000008u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_SSL_INTR_STATUS_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_RD_INTR_STATUS_MASK (0x00000200u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_RD_INTR_STATUS_SHIFT (0x00000009u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_RD_INTR_STATUS_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_DNT_INTR_STATUS_MASK (0x00000400u)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_DNT_INTR_STATUS_SHIFT (0x0000000Au)
#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_DNT_INTR_STATUS_RESETVAL (0x00000000u)


#define CSL_BCP_TM_BCP_INTERRUPT_STATUS_RESETVAL (0x00000000u)

/* bcp_halt_on_error */

#define CSL_BCP_TM_BCP_HALT_ON_ERROR_TM_HALT_ON_ERROR_MASK (0x00000001u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_TM_HALT_ON_ERROR_SHIFT (0x00000000u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_TM_HALT_ON_ERROR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_HALT_ON_ERROR_DIO_HALT_ON_ERROR_MASK (0x00000002u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_DIO_HALT_ON_ERROR_SHIFT (0x00000001u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_DIO_HALT_ON_ERROR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_HALT_ON_ERROR_CRC_HALT_ON_ERROR_MASK (0x00000004u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_CRC_HALT_ON_ERROR_SHIFT (0x00000002u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_CRC_HALT_ON_ERROR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_HALT_ON_ERROR_ENC_HALT_ON_ERROR_MASK (0x00000008u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_ENC_HALT_ON_ERROR_SHIFT (0x00000003u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_ENC_HALT_ON_ERROR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_HALT_ON_ERROR_RM_HALT_ON_ERROR_MASK (0x00000010u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_RM_HALT_ON_ERROR_SHIFT (0x00000004u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_RM_HALT_ON_ERROR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_HALT_ON_ERROR_MOD_HALT_ON_ERROR_MASK (0x00000020u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_MOD_HALT_ON_ERROR_SHIFT (0x00000005u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_MOD_HALT_ON_ERROR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_HALT_ON_ERROR_INT_HALT_ON_ERROR_MASK (0x00000040u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_INT_HALT_ON_ERROR_SHIFT (0x00000006u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_INT_HALT_ON_ERROR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_HALT_ON_ERROR_COR_HALT_ON_ERROR_MASK (0x00000080u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_COR_HALT_ON_ERROR_SHIFT (0x00000007u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_COR_HALT_ON_ERROR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_HALT_ON_ERROR_SSL_HALT_ON_ERROR_MASK (0x00000100u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_SSL_HALT_ON_ERROR_SHIFT (0x00000008u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_SSL_HALT_ON_ERROR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_HALT_ON_ERROR_RD_HALT_ON_ERROR_MASK (0x00000200u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_RD_HALT_ON_ERROR_SHIFT (0x00000009u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_RD_HALT_ON_ERROR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_HALT_ON_ERROR_DNT_HALT_ON_ERROR_MASK (0x00000400u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_DNT_HALT_ON_ERROR_SHIFT (0x0000000Au)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_DNT_HALT_ON_ERROR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_HALT_ON_ERROR_FRC_HALT_ON_ERROR_MASK (0x00000800u)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_FRC_HALT_ON_ERROR_SHIFT (0x0000000Bu)
#define CSL_BCP_TM_BCP_HALT_ON_ERROR_FRC_HALT_ON_ERROR_RESETVAL (0x00000000u)


#define CSL_BCP_TM_BCP_HALT_ON_ERROR_RESETVAL (0x00000000u)

/* TM_intr_irs */

#define CSL_BCP_TM_TM_INTR_IRS_PPB_ERR_MASK (0x00000001u)
#define CSL_BCP_TM_TM_INTR_IRS_PPB_ERR_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_INTR_IRS_PPB_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_INTR_IRS_DLG_DONE_INT_MASK (0x00000002u)
#define CSL_BCP_TM_TM_INTR_IRS_DLG_DONE_INT_SHIFT (0x00000001u)
#define CSL_BCP_TM_TM_INTR_IRS_DLG_DONE_INT_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_INTR_IRS_ENG_ERROR_MASK (0x000001FCu)
#define CSL_BCP_TM_TM_INTR_IRS_ENG_ERROR_SHIFT (0x00000002u)
#define CSL_BCP_TM_TM_INTR_IRS_ENG_ERROR_RESETVAL (0x00000000u)


#define CSL_BCP_TM_TM_INTR_IRS_RESETVAL  (0x00000000u)

/* TM_intr_irs_set */

#define CSL_BCP_TM_TM_INTR_IRS_SET_PPB_ERR_MASK (0x00000001u)
#define CSL_BCP_TM_TM_INTR_IRS_SET_PPB_ERR_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_INTR_IRS_SET_PPB_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_INTR_IRS_SET_DLG_DONE_INT_MASK (0x00000002u)
#define CSL_BCP_TM_TM_INTR_IRS_SET_DLG_DONE_INT_SHIFT (0x00000001u)
#define CSL_BCP_TM_TM_INTR_IRS_SET_DLG_DONE_INT_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_INTR_IRS_SET_ENG_ERROR_MASK (0x000001FCu)
#define CSL_BCP_TM_TM_INTR_IRS_SET_ENG_ERROR_SHIFT (0x00000002u)
#define CSL_BCP_TM_TM_INTR_IRS_SET_ENG_ERROR_RESETVAL (0x00000000u)


#define CSL_BCP_TM_TM_INTR_IRS_SET_RESETVAL (0x00000000u)

/* TM_intr_irs_clr */

#define CSL_BCP_TM_TM_INTR_IRS_CLR_PPB_ERR_MASK (0x00000001u)
#define CSL_BCP_TM_TM_INTR_IRS_CLR_PPB_ERR_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_INTR_IRS_CLR_PPB_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_INTR_IRS_CLR_DLG_DONE_INT_MASK (0x00000002u)
#define CSL_BCP_TM_TM_INTR_IRS_CLR_DLG_DONE_INT_SHIFT (0x00000001u)
#define CSL_BCP_TM_TM_INTR_IRS_CLR_DLG_DONE_INT_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_INTR_IRS_CLR_ENG_ERROR_MASK (0x000001FCu)
#define CSL_BCP_TM_TM_INTR_IRS_CLR_ENG_ERROR_SHIFT (0x00000002u)
#define CSL_BCP_TM_TM_INTR_IRS_CLR_ENG_ERROR_RESETVAL (0x00000000u)


#define CSL_BCP_TM_TM_INTR_IRS_CLR_RESETVAL (0x00000000u)

/* TM_intr_en */

#define CSL_BCP_TM_TM_INTR_EN_PPB_ERR_MASK (0x00000001u)
#define CSL_BCP_TM_TM_INTR_EN_PPB_ERR_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_INTR_EN_PPB_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_INTR_EN_DLG_DONE_INT_MASK (0x00000002u)
#define CSL_BCP_TM_TM_INTR_EN_DLG_DONE_INT_SHIFT (0x00000001u)
#define CSL_BCP_TM_TM_INTR_EN_DLG_DONE_INT_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_INTR_EN_ENG_ERROR_MASK (0x000001FCu)
#define CSL_BCP_TM_TM_INTR_EN_ENG_ERROR_SHIFT (0x00000002u)
#define CSL_BCP_TM_TM_INTR_EN_ENG_ERROR_RESETVAL (0x00000000u)


#define CSL_BCP_TM_TM_INTR_EN_RESETVAL   (0x00000000u)

/* TM_intr_en_set */

#define CSL_BCP_TM_TM_INTR_EN_SET_PPB_ERR_MASK (0x00000001u)
#define CSL_BCP_TM_TM_INTR_EN_SET_PPB_ERR_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_INTR_EN_SET_PPB_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_INTR_EN_SET_DLG_DONE_INT_MASK (0x00000002u)
#define CSL_BCP_TM_TM_INTR_EN_SET_DLG_DONE_INT_SHIFT (0x00000001u)
#define CSL_BCP_TM_TM_INTR_EN_SET_DLG_DONE_INT_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_INTR_EN_SET_ENG_ERROR_MASK (0x000001FCu)
#define CSL_BCP_TM_TM_INTR_EN_SET_ENG_ERROR_SHIFT (0x00000002u)
#define CSL_BCP_TM_TM_INTR_EN_SET_ENG_ERROR_RESETVAL (0x00000000u)


#define CSL_BCP_TM_TM_INTR_EN_SET_RESETVAL (0x00000000u)

/* TM_intr_en_clr */

#define CSL_BCP_TM_TM_INTR_EN_CLR_PPB_ERR_MASK (0x00000001u)
#define CSL_BCP_TM_TM_INTR_EN_CLR_PPB_ERR_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_INTR_EN_CLR_PPB_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_INTR_EN_CLR_DLG_DONE_INT_MASK (0x00000002u)
#define CSL_BCP_TM_TM_INTR_EN_CLR_DLG_DONE_INT_SHIFT (0x00000001u)
#define CSL_BCP_TM_TM_INTR_EN_CLR_DLG_DONE_INT_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_INTR_EN_CLR_ENG_ERROR_MASK (0x000001FCu)
#define CSL_BCP_TM_TM_INTR_EN_CLR_ENG_ERROR_SHIFT (0x00000002u)
#define CSL_BCP_TM_TM_INTR_EN_CLR_ENG_ERROR_RESETVAL (0x00000000u)


#define CSL_BCP_TM_TM_INTR_EN_CLR_RESETVAL (0x00000000u)

/* TM_intr_en_sts */

#define CSL_BCP_TM_TM_INTR_EN_STS_PPB_ERR_MASK (0x00000001u)
#define CSL_BCP_TM_TM_INTR_EN_STS_PPB_ERR_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_INTR_EN_STS_PPB_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_INTR_EN_STS_DLG_DONE_INT_MASK (0x00000002u)
#define CSL_BCP_TM_TM_INTR_EN_STS_DLG_DONE_INT_SHIFT (0x00000001u)
#define CSL_BCP_TM_TM_INTR_EN_STS_DLG_DONE_INT_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_INTR_EN_STS_ENG_ERROR_MASK (0x000001FCu)
#define CSL_BCP_TM_TM_INTR_EN_STS_ENG_ERROR_SHIFT (0x00000002u)
#define CSL_BCP_TM_TM_INTR_EN_STS_ENG_ERROR_RESETVAL (0x00000000u)


#define CSL_BCP_TM_TM_INTR_EN_STS_RESETVAL (0x00000000u)

/* tm_dlg_global_hdr */

#define CSL_BCP_TM_TM_DLG_GLOBAL_HDR_GLOBAL_HDR_MASK (0xFFFFFFFFu)
#define CSL_BCP_TM_TM_DLG_GLOBAL_HDR_GLOBAL_HDR_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_DLG_GLOBAL_HDR_GLOBAL_HDR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_DLG_GLOBAL_HDR_RESETVAL (0x00000000u)

/* tm_dlg_tm_sw_timestamp */

#define CSL_BCP_TM_TM_DLG_TM_SW_TIMESTAMP_TM_SW_TIMESTAMP_MASK (0xFFFFFFFFu)
#define CSL_BCP_TM_TM_DLG_TM_SW_TIMESTAMP_TM_SW_TIMESTAMP_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_DLG_TM_SW_TIMESTAMP_TM_SW_TIMESTAMP_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_DLG_TM_SW_TIMESTAMP_RESETVAL (0x00000000u)

/* tm_dlg_hw_timestamp_eng_err */

#define CSL_BCP_TM_TM_DLG_HW_TIMESTAMP_ENG_ERR_TM_HW_TIMESTAMP_MASK (0x00FFFFFFu)
#define CSL_BCP_TM_TM_DLG_HW_TIMESTAMP_ENG_ERR_TM_HW_TIMESTAMP_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_DLG_HW_TIMESTAMP_ENG_ERR_TM_HW_TIMESTAMP_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_DLG_HW_TIMESTAMP_ENG_ERR_ENG_ERR_MASK (0xFF000000u)
#define CSL_BCP_TM_TM_DLG_HW_TIMESTAMP_ENG_ERR_ENG_ERR_SHIFT (0x00000018u)
#define CSL_BCP_TM_TM_DLG_HW_TIMESTAMP_ENG_ERR_ENG_ERR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_DLG_HW_TIMESTAMP_ENG_ERR_RESETVAL (0x00000000u)

/* bcp_pid */

#define CSL_BCP_TM_BCP_PID_BCP_PID_MINOR_MASK (0x0000003Fu)
#define CSL_BCP_TM_BCP_PID_BCP_PID_MINOR_SHIFT (0x00000000u)
#define CSL_BCP_TM_BCP_PID_BCP_PID_MINOR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_PID_BCP_PID_CUSTOM_MASK (0x000000C0u)
#define CSL_BCP_TM_BCP_PID_BCP_PID_CUSTOM_SHIFT (0x00000006u)
#define CSL_BCP_TM_BCP_PID_BCP_PID_CUSTOM_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_PID_BCP_PID_MAJOR_MASK (0x00000700u)
#define CSL_BCP_TM_BCP_PID_BCP_PID_MAJOR_SHIFT (0x00000008u)
#define CSL_BCP_TM_BCP_PID_BCP_PID_MAJOR_RESETVAL (0x00000001u)

#define CSL_BCP_TM_BCP_PID_BCP_PID_RTL_MASK (0x0000F800u)
#define CSL_BCP_TM_BCP_PID_BCP_PID_RTL_SHIFT (0x0000000Bu)
#define CSL_BCP_TM_BCP_PID_BCP_PID_RTL_RESETVAL (0x0000000Fu)

#define CSL_BCP_TM_BCP_PID_BCP_PID_FUNC_MASK (0x0FFF0000u)
#define CSL_BCP_TM_BCP_PID_BCP_PID_FUNC_SHIFT (0x00000010u)
#define CSL_BCP_TM_BCP_PID_BCP_PID_FUNC_RESETVAL (0x00000810u)


#define CSL_BCP_TM_BCP_PID_BCP_PID_SCHEME_MASK (0xC0000000u)
#define CSL_BCP_TM_BCP_PID_BCP_PID_SCHEME_SHIFT (0x0000001Eu)
#define CSL_BCP_TM_BCP_PID_BCP_PID_SCHEME_RESETVAL (0x00000001u)

#define CSL_BCP_TM_BCP_PID_RESETVAL      (0x48107900u)

/* bcp_soft_reset */

#define CSL_BCP_TM_BCP_SOFT_RESET_SOFT_RESET_MASK (0x00000001u)
#define CSL_BCP_TM_BCP_SOFT_RESET_SOFT_RESET_SHIFT (0x00000000u)
#define CSL_BCP_TM_BCP_SOFT_RESET_SOFT_RESET_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_SOFT_RESET_RX_TEARDOWN_MASK (0x00000002u)
#define CSL_BCP_TM_BCP_SOFT_RESET_RX_TEARDOWN_SHIFT (0x00000001u)
#define CSL_BCP_TM_BCP_SOFT_RESET_RX_TEARDOWN_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_SOFT_RESET_TX_TEARDOWN_MASK (0x00000004u)
#define CSL_BCP_TM_BCP_SOFT_RESET_TX_TEARDOWN_SHIFT (0x00000002u)
#define CSL_BCP_TM_BCP_SOFT_RESET_TX_TEARDOWN_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_SOFT_RESET_CLKGATE_DISABLE_MASK (0x00000008u)
#define CSL_BCP_TM_BCP_SOFT_RESET_CLKGATE_DISABLE_SHIFT (0x00000003u)
#define CSL_BCP_TM_BCP_SOFT_RESET_CLKGATE_DISABLE_RESETVAL (0x00000000u)


#define CSL_BCP_TM_BCP_SOFT_RESET_RESETVAL (0x00000000u)

/* tm_halt_ctrl */

#define CSL_BCP_TM_TM_HALT_CTRL_CLEAR_HALT_MASK (0x000000FFu)
#define CSL_BCP_TM_TM_HALT_CTRL_CLEAR_HALT_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_HALT_CTRL_CLEAR_HALT_RESETVAL (0x00000000u)


#define CSL_BCP_TM_TM_HALT_CTRL_RESETVAL (0x00000000u)

/* tm_halt_status */

#define CSL_BCP_TM_TM_HALT_STATUS_HALT_STATUS_MASK (0x000000FFu)
#define CSL_BCP_TM_TM_HALT_STATUS_HALT_STATUS_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_HALT_STATUS_HALT_STATUS_RESETVAL (0x00000000u)


#define CSL_BCP_TM_TM_HALT_STATUS_RESETVAL (0x00000000u)

/* bcp_emu_clkstop_ctrl */

#define CSL_BCP_TM_BCP_EMU_CLKSTOP_CTRL_BCP_EMU_FREERUN_MASK (0x00000001u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_CTRL_BCP_EMU_FREERUN_SHIFT (0x00000000u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_CTRL_BCP_EMU_FREERUN_RESETVAL (0x00000000u)


#define CSL_BCP_TM_BCP_EMU_CLKSTOP_CTRL_BCP_EMU_RT_SEL_MASK (0x00000004u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_CTRL_BCP_EMU_RT_SEL_SHIFT (0x00000002u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_CTRL_BCP_EMU_RT_SEL_RESETVAL (0x00000000u)


#define CSL_BCP_TM_BCP_EMU_CLKSTOP_CTRL_RESETVAL (0x00000000u)

/* bcp_emu_clkstop_status */

#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_TM_ENG_IDLE_MASK (0x00000001u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_TM_ENG_IDLE_SHIFT (0x00000000u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_TM_ENG_IDLE_RESETVAL (0x00000001u)

#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_DIO_ENG_IDLE_MASK (0x00000002u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_DIO_ENG_IDLE_SHIFT (0x00000001u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_DIO_ENG_IDLE_RESETVAL (0x00000001u)

#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_CRC_ENG_IDLE_MASK (0x00000004u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_CRC_ENG_IDLE_SHIFT (0x00000002u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_CRC_ENG_IDLE_RESETVAL (0x00000001u)

#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_ENC_ENG_IDLE_MASK (0x00000008u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_ENC_ENG_IDLE_SHIFT (0x00000003u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_ENC_ENG_IDLE_RESETVAL (0x00000001u)

#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_RM_ENG_IDLE_MASK (0x00000010u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_RM_ENG_IDLE_SHIFT (0x00000004u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_RM_ENG_IDLE_RESETVAL (0x00000001u)

#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_MOD_ENG_IDLE_MASK (0x00000020u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_MOD_ENG_IDLE_SHIFT (0x00000005u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_MOD_ENG_IDLE_RESETVAL (0x00000001u)

#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_INT_ENG_IDLE_MASK (0x00000040u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_INT_ENG_IDLE_SHIFT (0x00000006u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_INT_ENG_IDLE_RESETVAL (0x00000001u)

#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_COR_ENG_IDLE_MASK (0x00000080u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_COR_ENG_IDLE_SHIFT (0x00000007u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_COR_ENG_IDLE_RESETVAL (0x00000001u)

#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_SSL_ENG_IDLE_MASK (0x00000100u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_SSL_ENG_IDLE_SHIFT (0x00000008u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_SSL_ENG_IDLE_RESETVAL (0x00000001u)

#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_RD_ENG_IDLE_MASK (0x00000200u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_RD_ENG_IDLE_SHIFT (0x00000009u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_RD_ENG_IDLE_RESETVAL (0x00000001u)

#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_DNT_ENG_IDLE_MASK (0x00000400u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_DNT_ENG_IDLE_SHIFT (0x0000000Au)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_DNT_ENG_IDLE_RESETVAL (0x00000001u)

#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_ALL_ENG_IDLE_MASK (0x00000800u)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_ALL_ENG_IDLE_SHIFT (0x0000000Bu)
#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_ALL_ENG_IDLE_RESETVAL (0x00000001u)


#define CSL_BCP_TM_BCP_EMU_CLKSTOP_STATUS_RESETVAL (0x00000FFFu)

/* bcp_dlg_sw_timestamp */

#define CSL_BCP_TM_BCP_DLG_SW_TIMESTAMP_BCP_DLG_SW_TIMESTAMP_MASK (0xFFFFFFFFu)
#define CSL_BCP_TM_BCP_DLG_SW_TIMESTAMP_BCP_DLG_SW_TIMESTAMP_SHIFT (0x00000000u)
#define CSL_BCP_TM_BCP_DLG_SW_TIMESTAMP_BCP_DLG_SW_TIMESTAMP_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_SW_TIMESTAMP_RESETVAL (0x00000000u)

/* bcp_dlg_hw_timestamp */

#define CSL_BCP_TM_BCP_DLG_HW_TIMESTAMP_BCP_DLG_HW_TIMESTAMP_MASK (0x00FFFFFFu)
#define CSL_BCP_TM_BCP_DLG_HW_TIMESTAMP_BCP_DLG_HW_TIMESTAMP_SHIFT (0x00000000u)
#define CSL_BCP_TM_BCP_DLG_HW_TIMESTAMP_BCP_DLG_HW_TIMESTAMP_RESETVAL (0x00000000u)


#define CSL_BCP_TM_BCP_DLG_HW_TIMESTAMP_RESETVAL (0x00000000u)

/* bcp_dlg_hw_timestamp_ctrl */

#define CSL_BCP_TM_BCP_DLG_HW_TIMESTAMP_CTRL_BCP_DLG_HW_TIMESTAMP_RUN_MASK (0x00000001u)
#define CSL_BCP_TM_BCP_DLG_HW_TIMESTAMP_CTRL_BCP_DLG_HW_TIMESTAMP_RUN_SHIFT (0x00000000u)
#define CSL_BCP_TM_BCP_DLG_HW_TIMESTAMP_CTRL_BCP_DLG_HW_TIMESTAMP_RUN_RESETVAL (0x00000000u)


#define CSL_BCP_TM_BCP_DLG_HW_TIMESTAMP_CTRL_RESETVAL (0x00000000u)

/* bcp_dlg_hold_rst */

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_TM_DLG_RST_MASK (0x00000001u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_TM_DLG_RST_SHIFT (0x00000000u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_TM_DLG_RST_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_DIO_DLG_RST_MASK (0x00000002u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_DIO_DLG_RST_SHIFT (0x00000001u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_DIO_DLG_RST_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_CRC_DLG_RST_MASK (0x00000004u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_CRC_DLG_RST_SHIFT (0x00000002u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_CRC_DLG_RST_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_ENC_DLG_RST_MASK (0x00000008u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_ENC_DLG_RST_SHIFT (0x00000003u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_ENC_DLG_RST_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_RM_DLG_RST_MASK (0x00000010u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_RM_DLG_RST_SHIFT (0x00000004u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_RM_DLG_RST_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_MOD_DLG_RST_MASK (0x00000020u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_MOD_DLG_RST_SHIFT (0x00000005u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_MOD_DLG_RST_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_INT_DLG_RST_MASK (0x00000040u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_INT_DLG_RST_SHIFT (0x00000006u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_INT_DLG_RST_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_COR_DLG_RST_MASK (0x00000080u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_COR_DLG_RST_SHIFT (0x00000007u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_COR_DLG_RST_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_SSL_DLG_RST_MASK (0x00000100u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_SSL_DLG_RST_SHIFT (0x00000008u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_SSL_DLG_RST_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_RD_DLG_RST_MASK (0x00000200u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_RD_DLG_RST_SHIFT (0x00000009u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_RD_DLG_RST_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_DNT_DLG_RST_MASK (0x00000400u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_DNT_DLG_RST_SHIFT (0x0000000Au)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_DNT_DLG_RST_RESETVAL (0x00000000u)


#define CSL_BCP_TM_BCP_DLG_HOLD_RST_TM_DLG_HOLD_MASK (0x00010000u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_TM_DLG_HOLD_SHIFT (0x00000010u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_TM_DLG_HOLD_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_DIO_DLG_HOLD_MASK (0x00020000u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_DIO_DLG_HOLD_SHIFT (0x00000011u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_DIO_DLG_HOLD_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_CRC_DLG_HOLD_MASK (0x00040000u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_CRC_DLG_HOLD_SHIFT (0x00000012u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_CRC_DLG_HOLD_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_ENC_DLG_HOLD_MASK (0x00080000u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_ENC_DLG_HOLD_SHIFT (0x00000013u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_ENC_DLG_HOLD_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_RM_DLG_HOLD_MASK (0x00100000u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_RM_DLG_HOLD_SHIFT (0x00000014u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_RM_DLG_HOLD_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_MOD_DLG_HOLD_MASK (0x00200000u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_MOD_DLG_HOLD_SHIFT (0x00000015u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_MOD_DLG_HOLD_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_INT_DLG_HOLD_MASK (0x00400000u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_INT_DLG_HOLD_SHIFT (0x00000016u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_INT_DLG_HOLD_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_COR_DLG_HOLD_MASK (0x00800000u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_COR_DLG_HOLD_SHIFT (0x00000017u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_COR_DLG_HOLD_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_SSL_DLG_HOLD_MASK (0x01000000u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_SSL_DLG_HOLD_SHIFT (0x00000018u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_SSL_DLG_HOLD_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_RD_DLG_HOLD_MASK (0x02000000u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_RD_DLG_HOLD_SHIFT (0x00000019u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_RD_DLG_HOLD_RESETVAL (0x00000000u)

#define CSL_BCP_TM_BCP_DLG_HOLD_RST_DNT_DLG_HOLD_MASK (0x04000000u)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_DNT_DLG_HOLD_SHIFT (0x0000001Au)
#define CSL_BCP_TM_BCP_DLG_HOLD_RST_DNT_DLG_HOLD_RESETVAL (0x00000000u)


#define CSL_BCP_TM_BCP_DLG_HOLD_RST_RESETVAL (0x00000000u)

/* tm_control */

#define CSL_BCP_TM_TM_CONTROL_TX_CDMAHP_RD_ARB_HPRIORITY_MASK (0x00000001u)
#define CSL_BCP_TM_TM_CONTROL_TX_CDMAHP_RD_ARB_HPRIORITY_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_CONTROL_TX_CDMAHP_RD_ARB_HPRIORITY_RESETVAL (0x00000001u)

#define CSL_BCP_TM_TM_CONTROL_FRC_PAYLOAD_ENDIAN_MASK (0x00000002u)
#define CSL_BCP_TM_TM_CONTROL_FRC_PAYLOAD_ENDIAN_SHIFT (0x00000001u)
#define CSL_BCP_TM_TM_CONTROL_FRC_PAYLOAD_ENDIAN_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_CONTROL_CDMAHP_DISABLE_MASK (0x00000004u)
#define CSL_BCP_TM_TM_CONTROL_CDMAHP_DISABLE_SHIFT (0x00000002u)
#define CSL_BCP_TM_TM_CONTROL_CDMAHP_DISABLE_RESETVAL (0x00000000u)


#define CSL_BCP_TM_TM_CONTROL_CDMAHP_SRC_ID_MASK (0x0000FF00u)
#define CSL_BCP_TM_TM_CONTROL_CDMAHP_SRC_ID_SHIFT (0x00000008u)
#define CSL_BCP_TM_TM_CONTROL_CDMAHP_SRC_ID_RESETVAL (0x00000000u)


#define CSL_BCP_TM_TM_CONTROL_RESETVAL   (0x00000001u)

/* tm_tx_cdmahp_read_priority */

#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_0_MASK (0x00000003u)
#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_0_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_0_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_1_MASK (0x0000000Cu)
#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_1_SHIFT (0x00000002u)
#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_1_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_2_MASK (0x00000030u)
#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_2_SHIFT (0x00000004u)
#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_2_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_3_MASK (0x000000C0u)
#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_3_SHIFT (0x00000006u)
#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_3_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_4_MASK (0x00000300u)
#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_4_SHIFT (0x00000008u)
#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_4_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_5_MASK (0x00000C00u)
#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_5_SHIFT (0x0000000Au)
#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_5_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_6_MASK (0x00003000u)
#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_6_SHIFT (0x0000000Cu)
#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_6_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_7_MASK (0x0000C000u)
#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_7_SHIFT (0x0000000Eu)
#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_TX_CDMAHP_RD_ARB_PRIORITY_7_RESETVAL (0x00000000u)


#define CSL_BCP_TM_TM_TX_CDMAHP_READ_PRIORITY_RESETVAL (0x00000000u)

/* tm_tx_qfifo_rd_dest_sel */

#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_0_MASK (0x00000003u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_0_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_0_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_1_MASK (0x0000000Cu)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_1_SHIFT (0x00000002u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_1_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_2_MASK (0x00000030u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_2_SHIFT (0x00000004u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_2_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_3_MASK (0x000000C0u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_3_SHIFT (0x00000006u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_3_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_4_MASK (0x00000300u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_4_SHIFT (0x00000008u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_4_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_5_MASK (0x00000C00u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_5_SHIFT (0x0000000Au)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_5_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_6_MASK (0x00003000u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_6_SHIFT (0x0000000Cu)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_6_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_7_MASK (0x0000C000u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_7_SHIFT (0x0000000Eu)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_ARB_PRIORITY_7_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_0_MASK (0x00030000u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_0_SHIFT (0x00000010u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_0_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_1_MASK (0x000C0000u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_1_SHIFT (0x00000012u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_1_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_2_MASK (0x00300000u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_2_SHIFT (0x00000014u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_2_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_3_MASK (0x00C00000u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_3_SHIFT (0x00000016u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_3_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_4_MASK (0x03000000u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_4_SHIFT (0x00000018u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_4_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_5_MASK (0x0C000000u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_5_SHIFT (0x0000001Au)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_5_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_6_MASK (0x30000000u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_6_SHIFT (0x0000001Cu)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_6_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_7_MASK (0xC0000000u)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_7_SHIFT (0x0000001Eu)
#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_TX_QFIFO_RD_DEST_SEL_7_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_TX_QFIFO_RD_DEST_SEL_RESETVAL (0x00000000u)

/* tm_rx_qfifo_wr_arb_priority_3_0 */

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_0_WR_ARB_PRIORITY_0_MASK (0x00000003u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_0_WR_ARB_PRIORITY_0_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_0_WR_ARB_PRIORITY_0_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_0_WR_ARB_PRIORITY_1_MASK (0x0000000Cu)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_0_WR_ARB_PRIORITY_1_SHIFT (0x00000002u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_0_WR_ARB_PRIORITY_1_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_0_WR_ARB_PRIORITY_2_MASK (0x00000030u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_0_WR_ARB_PRIORITY_2_SHIFT (0x00000004u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_0_WR_ARB_PRIORITY_2_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_0_WR_ARB_PRIORITY_3_MASK (0x000000C0u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_0_WR_ARB_PRIORITY_3_SHIFT (0x00000006u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_0_WR_ARB_PRIORITY_3_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_1_WR_ARB_PRIORITY_0_MASK (0x00000300u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_1_WR_ARB_PRIORITY_0_SHIFT (0x00000008u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_1_WR_ARB_PRIORITY_0_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_1_WR_ARB_PRIORITY_1_MASK (0x00000C00u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_1_WR_ARB_PRIORITY_1_SHIFT (0x0000000Au)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_1_WR_ARB_PRIORITY_1_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_1_WR_ARB_PRIORITY_2_MASK (0x00003000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_1_WR_ARB_PRIORITY_2_SHIFT (0x0000000Cu)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_1_WR_ARB_PRIORITY_2_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_1_WR_ARB_PRIORITY_3_MASK (0x0000C000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_1_WR_ARB_PRIORITY_3_SHIFT (0x0000000Eu)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_1_WR_ARB_PRIORITY_3_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_2_WR_ARB_PRIORITY_0_MASK (0x00030000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_2_WR_ARB_PRIORITY_0_SHIFT (0x00000010u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_2_WR_ARB_PRIORITY_0_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_2_WR_ARB_PRIORITY_1_MASK (0x000C0000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_2_WR_ARB_PRIORITY_1_SHIFT (0x00000012u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_2_WR_ARB_PRIORITY_1_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_2_WR_ARB_PRIORITY_2_MASK (0x00300000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_2_WR_ARB_PRIORITY_2_SHIFT (0x00000014u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_2_WR_ARB_PRIORITY_2_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_2_WR_ARB_PRIORITY_3_MASK (0x00C00000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_2_WR_ARB_PRIORITY_3_SHIFT (0x00000016u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_2_WR_ARB_PRIORITY_3_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_3_WR_ARB_PRIORITY_0_MASK (0x03000000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_3_WR_ARB_PRIORITY_0_SHIFT (0x00000018u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_3_WR_ARB_PRIORITY_0_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_3_WR_ARB_PRIORITY_1_MASK (0x0C000000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_3_WR_ARB_PRIORITY_1_SHIFT (0x0000001Au)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_3_WR_ARB_PRIORITY_1_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_3_WR_ARB_PRIORITY_2_MASK (0x30000000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_3_WR_ARB_PRIORITY_2_SHIFT (0x0000001Cu)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_3_WR_ARB_PRIORITY_2_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_3_WR_ARB_PRIORITY_3_MASK (0xC0000000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_3_WR_ARB_PRIORITY_3_SHIFT (0x0000001Eu)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_TX_QFIFO_3_WR_ARB_PRIORITY_3_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_3_0_RESETVAL (0x00000000u)

/* tm_rx_qfifo_wr_arb_priority_7_4 */

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_4_WR_ARB_PRIORITY_0_MASK (0x00000003u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_4_WR_ARB_PRIORITY_0_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_4_WR_ARB_PRIORITY_0_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_4_WR_ARB_PRIORITY_1_MASK (0x0000000Cu)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_4_WR_ARB_PRIORITY_1_SHIFT (0x00000002u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_4_WR_ARB_PRIORITY_1_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_4_WR_ARB_PRIORITY_2_MASK (0x00000030u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_4_WR_ARB_PRIORITY_2_SHIFT (0x00000004u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_4_WR_ARB_PRIORITY_2_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_4_WR_ARB_PRIORITY_3_MASK (0x000000C0u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_4_WR_ARB_PRIORITY_3_SHIFT (0x00000006u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_4_WR_ARB_PRIORITY_3_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_5_WR_ARB_PRIORITY_0_MASK (0x00000300u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_5_WR_ARB_PRIORITY_0_SHIFT (0x00000008u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_5_WR_ARB_PRIORITY_0_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_5_WR_ARB_PRIORITY_1_MASK (0x00000C00u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_5_WR_ARB_PRIORITY_1_SHIFT (0x0000000Au)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_5_WR_ARB_PRIORITY_1_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_5_WR_ARB_PRIORITY_2_MASK (0x00003000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_5_WR_ARB_PRIORITY_2_SHIFT (0x0000000Cu)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_5_WR_ARB_PRIORITY_2_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_5_WR_ARB_PRIORITY_3_MASK (0x0000C000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_5_WR_ARB_PRIORITY_3_SHIFT (0x0000000Eu)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_5_WR_ARB_PRIORITY_3_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_6_WR_ARB_PRIORITY_0_MASK (0x00030000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_6_WR_ARB_PRIORITY_0_SHIFT (0x00000010u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_6_WR_ARB_PRIORITY_0_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_6_WR_ARB_PRIORITY_1_MASK (0x000C0000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_6_WR_ARB_PRIORITY_1_SHIFT (0x00000012u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_6_WR_ARB_PRIORITY_1_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_6_WR_ARB_PRIORITY_2_MASK (0x00300000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_6_WR_ARB_PRIORITY_2_SHIFT (0x00000014u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_6_WR_ARB_PRIORITY_2_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_6_WR_ARB_PRIORITY_3_MASK (0x00C00000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_6_WR_ARB_PRIORITY_3_SHIFT (0x00000016u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_6_WR_ARB_PRIORITY_3_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_7_WR_ARB_PRIORITY_0_MASK (0x03000000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_7_WR_ARB_PRIORITY_0_SHIFT (0x00000018u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_7_WR_ARB_PRIORITY_0_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_7_WR_ARB_PRIORITY_1_MASK (0x0C000000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_7_WR_ARB_PRIORITY_1_SHIFT (0x0000001Au)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_7_WR_ARB_PRIORITY_1_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_7_WR_ARB_PRIORITY_2_MASK (0x30000000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_7_WR_ARB_PRIORITY_2_SHIFT (0x0000001Cu)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_7_WR_ARB_PRIORITY_2_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_7_WR_ARB_PRIORITY_3_MASK (0xC0000000u)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_7_WR_ARB_PRIORITY_3_SHIFT (0x0000001Eu)
#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_TX_QFIFO_7_WR_ARB_PRIORITY_3_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_QFIFO_WR_ARB_PRIORITY_7_4_RESETVAL (0x00000000u)

/* tm_rx_cdmahp_wr_arb priority */

#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_0_MASK (0x00000003u)
#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_0_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_0_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_1_MASK (0x0000000Cu)
#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_1_SHIFT (0x00000002u)
#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_1_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_2_MASK (0x00000030u)
#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_2_SHIFT (0x00000004u)
#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_2_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_3_MASK (0x000000C0u)
#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_3_SHIFT (0x00000006u)
#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_3_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_4_MASK (0x00000300u)
#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_4_SHIFT (0x00000008u)
#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_4_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_5_MASK (0x00000C00u)
#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_5_SHIFT (0x0000000Au)
#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_5_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_6_MASK (0x00003000u)
#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_6_SHIFT (0x0000000Cu)
#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_6_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_7_MASK (0x0000C000u)
#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_7_SHIFT (0x0000000Eu)
#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RX_CDMAHP_WR_ARB_PRIORITY_7_RESETVAL (0x00000000u)


#define CSL_BCP_TM_TM_RX_CDMAHP_WR_ARBPRIORITY_RESETVAL (0x00000000u)

/* bcp_eoi */

#define CSL_BCP_TM_BCP_EOI_BCP_EOI_MASK  (0x000000FFu)
#define CSL_BCP_TM_BCP_EOI_BCP_EOI_SHIFT (0x00000000u)
#define CSL_BCP_TM_BCP_EOI_BCP_EOI_RESETVAL (0x00000000u)


#define CSL_BCP_TM_BCP_EOI_RESETVAL      (0x00000000u)

/* cdma_desc_starve_status */

#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_0_MASK (0x00000001u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_0_SHIFT (0x00000000u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_0_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_1_MASK (0x00000002u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_1_SHIFT (0x00000001u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_1_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_2_MASK (0x00000004u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_2_SHIFT (0x00000002u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_2_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_3_MASK (0x00000008u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_3_SHIFT (0x00000003u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_3_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_4_MASK (0x00000010u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_4_SHIFT (0x00000004u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_4_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_5_MASK (0x00000020u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_5_SHIFT (0x00000005u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_5_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_6_MASK (0x00000040u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_6_SHIFT (0x00000006u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_6_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_7_MASK (0x00000080u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_7_SHIFT (0x00000007u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_DESC_STARVE_7_RESETVAL (0x00000000u)


#define CSL_BCP_TM_CDMA_DESC_STARVE_STATUS_RESETVAL (0x00000000u)

/* cdma_desc_starve_clear */

#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_0_MASK (0x00000001u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_0_SHIFT (0x00000000u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_0_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_1_MASK (0x00000002u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_1_SHIFT (0x00000001u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_1_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_2_MASK (0x00000004u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_2_SHIFT (0x00000002u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_2_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_3_MASK (0x00000008u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_3_SHIFT (0x00000003u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_3_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_4_MASK (0x00000010u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_4_SHIFT (0x00000004u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_4_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_5_MASK (0x00000020u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_5_SHIFT (0x00000005u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_5_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_6_MASK (0x00000040u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_6_SHIFT (0x00000006u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_6_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_7_MASK (0x00000080u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_7_SHIFT (0x00000007u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_DESC_STARVE_7_RESETVAL (0x00000000u)


#define CSL_BCP_TM_CDMA_DESC_STARVE_CLEAR_RESETVAL (0x00000000u)

/* cdma_desc_starve_set */

#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_0_MASK (0x00000001u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_0_SHIFT (0x00000000u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_0_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_1_MASK (0x00000002u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_1_SHIFT (0x00000001u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_1_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_2_MASK (0x00000004u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_2_SHIFT (0x00000002u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_2_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_3_MASK (0x00000008u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_3_SHIFT (0x00000003u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_3_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_4_MASK (0x00000010u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_4_SHIFT (0x00000004u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_4_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_5_MASK (0x00000020u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_5_SHIFT (0x00000005u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_5_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_6_MASK (0x00000040u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_6_SHIFT (0x00000006u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_6_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_7_MASK (0x00000080u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_7_SHIFT (0x00000007u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_DESC_STARVE_7_RESETVAL (0x00000000u)


#define CSL_BCP_TM_CDMA_DESC_STARVE_SET_RESETVAL (0x00000000u)

/* cdma_desc_starve_intr_sel */

#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_0_MASK (0x0000000Fu)
#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_0_SHIFT (0x00000000u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_0_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_1_MASK (0x000000F0u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_1_SHIFT (0x00000004u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_1_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_2_MASK (0x00000F00u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_2_SHIFT (0x00000008u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_2_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_3_MASK (0x0000F000u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_3_SHIFT (0x0000000Cu)
#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_3_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_4_MASK (0x000F0000u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_4_SHIFT (0x00000010u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_4_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_5_MASK (0x00F00000u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_5_SHIFT (0x00000014u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_5_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_6_MASK (0x0F000000u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_6_SHIFT (0x00000018u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_6_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_7_MASK (0xF0000000u)
#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_7_SHIFT (0x0000001Cu)
#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_DESC_STARVE_7_RESETVAL (0x00000000u)

#define CSL_BCP_TM_CDMA_DESC_STARVE_INTR_SEL_RESETVAL (0x00000000u)

/* TM_data_logger_ctl */

#define CSL_BCP_TM_TM_DATA_LOGGER_CTL_DLG_CTL_MASK (0x00000007u)
#define CSL_BCP_TM_TM_DATA_LOGGER_CTL_DLG_CTL_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_DATA_LOGGER_CTL_DLG_CTL_RESETVAL (0x00000000u)


#define CSL_BCP_TM_TM_DATA_LOGGER_CTL_DLG_ERR_MASK_MASK (0x01FF0000u)
#define CSL_BCP_TM_TM_DATA_LOGGER_CTL_DLG_ERR_MASK_SHIFT (0x00000010u)
#define CSL_BCP_TM_TM_DATA_LOGGER_CTL_DLG_ERR_MASK_RESETVAL (0x00000000u)


#define CSL_BCP_TM_TM_DATA_LOGGER_CTL_RESETVAL (0x00000000u)

/* TM_data_logger_status */

#define CSL_BCP_TM_TM_DATA_LOGGER_STATUS_DLG_WR_PTR_MASK (0x000000FFu)
#define CSL_BCP_TM_TM_DATA_LOGGER_STATUS_DLG_WR_PTR_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_DATA_LOGGER_STATUS_DLG_WR_PTR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_DATA_LOGGER_STATUS_DLG_WR_WRAP_MASK (0x0000FF00u)
#define CSL_BCP_TM_TM_DATA_LOGGER_STATUS_DLG_WR_WRAP_SHIFT (0x00000008u)
#define CSL_BCP_TM_TM_DATA_LOGGER_STATUS_DLG_WR_WRAP_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_DATA_LOGGER_STATUS_DLG_MEM_HAS_ERROR_MASK (0x00010000u)
#define CSL_BCP_TM_TM_DATA_LOGGER_STATUS_DLG_MEM_HAS_ERROR_SHIFT (0x00000010u)
#define CSL_BCP_TM_TM_DATA_LOGGER_STATUS_DLG_MEM_HAS_ERROR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_DATA_LOGGER_STATUS_DLG_RUNNING_MASK (0x00020000u)
#define CSL_BCP_TM_TM_DATA_LOGGER_STATUS_DLG_RUNNING_SHIFT (0x00000011u)
#define CSL_BCP_TM_TM_DATA_LOGGER_STATUS_DLG_RUNNING_RESETVAL (0x00000000u)


#define CSL_BCP_TM_TM_DATA_LOGGER_STATUS_RESETVAL (0x00000000u)

/* TM_global_hdr */

#define CSL_BCP_TM_TM_GLOBAL_HDR_GLOBAL_HDR_MASK (0xFFFFFFFFu)
#define CSL_BCP_TM_TM_GLOBAL_HDR_GLOBAL_HDR_SHIFT (0x00000000u)
#define CSL_BCP_TM_TM_GLOBAL_HDR_GLOBAL_HDR_RESETVAL (0x00000000u)

#define CSL_BCP_TM_TM_GLOBAL_HDR_RESETVAL (0x00000000u)

/* flow_id_table_0 */

#define CSL_BCP_TM_FLOW_ID_TABLE_0_ENDIAN_IN_MASK (0x00000003u)
#define CSL_BCP_TM_FLOW_ID_TABLE_0_ENDIAN_IN_SHIFT (0x00000000u)
#define CSL_BCP_TM_FLOW_ID_TABLE_0_ENDIAN_IN_RESETVAL (0x00000000u)

#define CSL_BCP_TM_FLOW_ID_TABLE_0_FORMAT_IN_MASK (0x0000001Cu)
#define CSL_BCP_TM_FLOW_ID_TABLE_0_FORMAT_IN_SHIFT (0x00000002u)
#define CSL_BCP_TM_FLOW_ID_TABLE_0_FORMAT_IN_RESETVAL (0x00000000u)


#define CSL_BCP_TM_FLOW_ID_TABLE_0_PKT_TYPE_MASK (0x00000F80u)
#define CSL_BCP_TM_FLOW_ID_TABLE_0_PKT_TYPE_SHIFT (0x00000007u)
#define CSL_BCP_TM_FLOW_ID_TABLE_0_PKT_TYPE_RESETVAL (0x00000000u)

#define CSL_BCP_TM_FLOW_ID_TABLE_0_DSP_INT_SEL_MASK (0x0000F000u)
#define CSL_BCP_TM_FLOW_ID_TABLE_0_DSP_INT_SEL_SHIFT (0x0000000Cu)
#define CSL_BCP_TM_FLOW_ID_TABLE_0_DSP_INT_SEL_RESETVAL (0x00000000u)

#define CSL_BCP_TM_FLOW_ID_TABLE_0_ENDIAN_OUT_MASK (0x00030000u)
#define CSL_BCP_TM_FLOW_ID_TABLE_0_ENDIAN_OUT_SHIFT (0x00000010u)
#define CSL_BCP_TM_FLOW_ID_TABLE_0_ENDIAN_OUT_RESETVAL (0x00000000u)

#define CSL_BCP_TM_FLOW_ID_TABLE_0_FORMAT_OUT_MASK (0x001C0000u)
#define CSL_BCP_TM_FLOW_ID_TABLE_0_FORMAT_OUT_SHIFT (0x00000012u)
#define CSL_BCP_TM_FLOW_ID_TABLE_0_FORMAT_OUT_RESETVAL (0x00000000u)

#define CSL_BCP_TM_FLOW_ID_TABLE_0_QFIFO_OUT_MASK (0x00E00000u)
#define CSL_BCP_TM_FLOW_ID_TABLE_0_QFIFO_OUT_SHIFT (0x00000015u)
#define CSL_BCP_TM_FLOW_ID_TABLE_0_QFIFO_OUT_RESETVAL (0x00000000u)

#define CSL_BCP_TM_FLOW_ID_TABLE_0_PS_FLAGS_MASK (0x0F000000u)
#define CSL_BCP_TM_FLOW_ID_TABLE_0_PS_FLAGS_SHIFT (0x00000018u)
#define CSL_BCP_TM_FLOW_ID_TABLE_0_PS_FLAGS_RESETVAL (0x00000000u)


#define CSL_BCP_TM_FLOW_ID_TABLE_0_RESETVAL (0x00000000u)

#endif
