/****************************************************************************\
 *           (C) Copyright 2009, Texas Instruments, Inc.                    *
 *                                                                          *
 *  Redistribution and use in source and binary forms, with or without      *
 *  modification, are permitted provided that the following conditions      *
 *  are met:                                                                *
 *                                                                          *
 *    Redistributions of source code must retain the above copyright        *
 *    notice, this list of conditions and the following disclaimer.         *
 *                                                                          *
 *    Redistributions in binary form must reproduce the above copyright     *
 *    notice, this list of conditions and the following disclaimer in the   *
 *    documentation and/or other materials provided with the                *
 *    distribution.                                                         *
 *                                                                          *
 *    Neither the name of Texas Instruments Incorporated nor the names of   *
 *    its contributors may be used to endorse or promote products derived   *
 *    from this software without specific prior written permission.         *
 *                                                                          *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS     *
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       *
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR   *
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT    *
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   *
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        *
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   *
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   *
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     *
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 ****************************************************************************
 *                                                                          *
 * Target processors : TMS320C66xx                                          *
 *                                                                          *
\****************************************************************************/
                    
/** 
 * @file AIF_init_dat.h
 *
 * @brief Header file for AIF H/W global data structures.
 * 
*/


/** @addtogroup AIF AIF
 *  @{
 */


#ifndef __AIF_INIT_DAT_H
#define __AIF_INIT_DAT_H

#include <ti/csl/csl.h>
#include <ti/csl/csl_aif2.h>
//#include <csl_fsync.h>
#include <ti/drv/aif2/AIF_defs.h>

#ifdef __cplusplus
extern "C" {
#endif

/*******************************************************************************
 * AIF global data declarations
 ******************************************************************************/


/**
 * @brief This flag indicates whether the event synchronization variables
 * have been initialized.
 */
extern Uint32 aifFsyncInitDone;

/**
 * @brief This flag indicates whether the Antenna Interface
 * has been initialized.
 */
extern Uint32 aifInitDone;

/**
 * @brief This structure points to the antenna interface registers
 * and is used to access them within the ABTLib.
 */
extern CSL_Aif2Regs * 	aifRegs;

/**
 * @brief This array counts AIF2 AT events and is updated
 * from AIF2 AT event cpu interrupts.
 * aifFsyncEventCount[1]: 10ms tick - AT event 7
 * aifFsyncEventCount[2-7]: debug sync events for ABT ingress EDMA events - AT event 0 to 5
 */
extern volatile Int32 aifFsyncEventCount[30];

extern 
volatile Uint32 aif2DioIntCount[AIF_MAX_NUM_LINKS];

#ifdef __cplusplus
}
#endif

#endif //__AIF_INIT_DAT_H

/** @} */ // end of module additions
